"""Config for pysma documentation."""

project = "PySMA"
copyright = "2021, Johann Kellerman, René Klomp"  # noqa: A001
author = "Johann Kellerman, René Klomp"

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
]

autoclass_content = "both"
html_theme = "sphinx_rtd_theme"



"""Helper functions for the pysma library."""

from dataclasses import dataclass
from typing import Any


def version_int_to_string(version_integer: Any) -> str:
    """Convert a version integer to a readable string.

    Args:
        version_integer (int): The version integer, as retrieved from the device.

    Returns:
        str: The version translated to a readable string.

    """
    if not version_integer:
        return ""
    if not isinstance(version_integer, int):
        return str(version_integer)

    appendixes = ["N", "E", "A", "B", "R", "S"]
    version_bytes = version_integer.to_bytes(4, "big")
    version_appendix = (
        appendixes[version_bytes[3]] if 0 <= version_bytes[3] < len(appendixes) else ""
    )
    return f"{version_bytes[0]:x}.{version_bytes[1]:x}.{version_bytes[2]}.{version_appendix}"


def ensure_string(value: Any) -> str:
    """Ensure the value is a string."""
    return str(value) if value is not None else ""


@dataclass(slots=True)
class DeviceInfo:
    """Device information."""

    serial: str = ""
    name: str = ""
    type: str = ""
    manufacturer: str = ""
    sw_version: str = ""

    def __post_init__(self) -> None:
        """Fallback values."""
        self.manufacturer = ensure_string(self.manufacturer) or "SMA"
        self.name = ensure_string(self.name) or "SMA Device"
        self.serial = ensure_string(self.serial) or "9999999999"
        self.sw_version = ensure_string(self.sw_version) or "0.0.0.E"
        self.type = ensure_string(self.type)



"""Test pysma helpers file."""

from dataclasses import asdict

from pysma.helpers import DeviceInfo, ensure_string, version_int_to_string


def test_version_int_to_string() -> None:
    """Ensure version_int_to_string returns correct values."""
    assert version_int_to_string(4294967294) == "ff.ff.255."
    assert version_int_to_string(51387396) == "3.10.28.R"
    assert version_int_to_string(1) == "0.0.0.E"
    assert version_int_to_string(0) == ""
    assert version_int_to_string(None) == ""  # type: ignore[arg-type]


def test_devtype() -> None:
    """Test DeviceInfo."""
    res = DeviceInfo(
        serial="123456789",
        name="SMA Device",
        type=ensure_string(None),
        manufacturer="SMA",
        sw_version="1.0.0",
    )
    assert asdict(res) == {
        "serial": "123456789",
        "name": "SMA Device",
        "type": "",
        "manufacturer": "SMA",
        "sw_version": "1.0.0",
    }

    res = DeviceInfo(sw_version="1")

    assert asdict(res) == {
        "serial": "9999999999",
        "name": "SMA Device",
        "type": "",
        "manufacturer": "SMA",
        "sw_version": "1",
    }



#!/usr/bin/env python
"""Basic usage example and testing of pysma."""

import argparse
import asyncio
import logging
import signal
import sys
from dataclasses import asdict
from typing import Any

import aiohttp

from pysma import exceptions, sma_webconnect

# This example will work with Python 3.9+

_LOG = logging.getLogger(__name__)

VAR = {}


def print_table(sensors: sma_webconnect.Sensors) -> None:
    """Print sensors formatted as table."""
    for sen in sensors:
        if sen.value is None:
            print(f"{sen.name:>25}")
        else:
            print(f"{sen.name:>25}{sen.value!s:>15} {sen.unit}")


async def main_loop(password: str, user: str, url: str) -> None:
    """Run main loop."""
    async with aiohttp.ClientSession(
        connector=aiohttp.TCPConnector(ssl=False)
    ) as session:
        VAR["sma"] = sma_webconnect.SMAWebConnect(
            session, url, password=password, group=user
        )

        try:
            await VAR["sma"].new_session()
        except exceptions.SmaAuthenticationException:
            _LOG.warning("Authentication failed!")
            return
        except exceptions.SmaConnectionException:
            _LOG.warning("Unable to connect to device at %s", url)
            return

        # We should not get any exceptions, but if we do we will close the session.
        try:
            VAR["running"] = True  # type: ignore[assignment]
            cnt = 5
            sensors = await VAR["sma"].get_sensors()
            device_info = await VAR["sma"].device_info()

            for name, value in asdict(device_info).items():
                print(f"{name:>15}{value:>25}")

            # enable all sensors
            for sensor in sensors:
                sensor.enabled = True

            while VAR.get("running"):
                await VAR["sma"].read(sensors)
                print_table(sensors)
                cnt -= 1
                if cnt == 0:
                    break
                await asyncio.sleep(2)
        finally:
            _LOG.info("Closing Session...")
            await VAR["sma"].close_session()


async def main() -> None:
    """Run example."""
    logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

    parser = argparse.ArgumentParser(description="Test the SMA webconnect library.")
    parser.add_argument(
        "url",
        type=str,
        help="Web address of the Webconnect module (http://ip-address or https://ip-address)",
    )
    parser.add_argument("user", choices=["user", "installer"], help="Login username")
    parser.add_argument("password", help="Login password")

    args = parser.parse_args()

    def _shutdown(*_: Any) -> None:
        VAR["running"] = False  # type: ignore[assignment]

    signal.signal(signal.SIGINT, _shutdown)

    await main_loop(user=args.user, password=args.password, url=args.url)


if __name__ == "__main__":
    asyncio.run(main())



"""PySMA library."""

from pysma.exceptions import (
    SmaAuthenticationException,
    SmaConnectionException,
    SmaException,
    SmaReadException,
)
from pysma.sensor import Sensor, Sensors
from pysma.sma_webconnect import SMAWebConnect

__all__ = [
    "SMAWebConnect",
    "Sensor",
    "Sensors",
    "SmaAuthenticationException",
    "SmaConnectionException",
    "SmaException",
    "SmaReadException",
]
